# coding: utf-8

"""
    Asana

    This is the interface for interacting with the [Asana Platform](https://developers.asana.com). Our API reference is generated from our [OpenAPI spec] (https://raw.githubusercontent.com/Asana/openapi/master/defs/asana_oas.yaml).  # noqa: E501

    OpenAPI spec version: 1.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class StoryResponse(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'gid': 'str',
        'resource_type': 'str',
        'created_at': 'datetime',
        'resource_subtype': 'str',
        'text': 'str',
        'html_text': 'str',
        'is_pinned': 'bool',
        'sticker_name': 'str',
        'created_by': 'CustomFieldResponsePeopleValue',
        'type': 'str',
        'is_editable': 'bool',
        'is_edited': 'bool',
        'hearted': 'bool',
        'hearts': 'list[GoalResponseLikes]',
        'num_hearts': 'int',
        'liked': 'bool',
        'likes': 'list[GoalResponseLikes]',
        'num_likes': 'int',
        'previews': 'list[StoryResponsePreviews]',
        'old_name': 'str',
        'new_name': 'str',
        'old_dates': 'StoryResponseOldDates',
        'new_dates': 'StoryResponseOldDates',
        'old_resource_subtype': 'str',
        'new_resource_subtype': 'str',
        'story': 'StoryResponseStory',
        'assignee': 'StoryResponseAssignee',
        'follower': 'StoryResponseAssignee',
        'old_section': 'StoryResponseOldSection',
        'new_section': 'StoryResponseOldSection',
        'task': 'StoryResponseTask',
        'project': 'StoryResponseProject',
        'tag': 'StoryResponseTag',
        'custom_field': 'StoryResponseCustomField',
        'old_text_value': 'str',
        'new_text_value': 'str',
        'old_number_value': 'int',
        'new_number_value': 'int',
        'old_enum_value': 'StoryResponseOldEnumValue',
        'new_enum_value': 'StoryResponseOldEnumValue',
        'old_date_value': 'AllOfStoryResponseOldDateValue',
        'new_date_value': 'AllOfStoryResponseNewDateValue',
        'old_people_value': 'list[CustomFieldResponsePeopleValue]',
        'new_people_value': 'list[CustomFieldResponsePeopleValue]',
        'old_multi_enum_values': 'list[CustomFieldBaseEnumOptions]',
        'new_multi_enum_values': 'list[CustomFieldBaseEnumOptions]',
        'new_approval_status': 'str',
        'old_approval_status': 'str',
        'duplicate_of': 'StoryResponseTask',
        'duplicated_from': 'StoryResponseTask',
        'dependency': 'StoryResponseTask',
        'source': 'str',
        'target': 'StoryResponseTarget'
    }

    attribute_map = {
        'gid': 'gid',
        'resource_type': 'resource_type',
        'created_at': 'created_at',
        'resource_subtype': 'resource_subtype',
        'text': 'text',
        'html_text': 'html_text',
        'is_pinned': 'is_pinned',
        'sticker_name': 'sticker_name',
        'created_by': 'created_by',
        'type': 'type',
        'is_editable': 'is_editable',
        'is_edited': 'is_edited',
        'hearted': 'hearted',
        'hearts': 'hearts',
        'num_hearts': 'num_hearts',
        'liked': 'liked',
        'likes': 'likes',
        'num_likes': 'num_likes',
        'previews': 'previews',
        'old_name': 'old_name',
        'new_name': 'new_name',
        'old_dates': 'old_dates',
        'new_dates': 'new_dates',
        'old_resource_subtype': 'old_resource_subtype',
        'new_resource_subtype': 'new_resource_subtype',
        'story': 'story',
        'assignee': 'assignee',
        'follower': 'follower',
        'old_section': 'old_section',
        'new_section': 'new_section',
        'task': 'task',
        'project': 'project',
        'tag': 'tag',
        'custom_field': 'custom_field',
        'old_text_value': 'old_text_value',
        'new_text_value': 'new_text_value',
        'old_number_value': 'old_number_value',
        'new_number_value': 'new_number_value',
        'old_enum_value': 'old_enum_value',
        'new_enum_value': 'new_enum_value',
        'old_date_value': 'old_date_value',
        'new_date_value': 'new_date_value',
        'old_people_value': 'old_people_value',
        'new_people_value': 'new_people_value',
        'old_multi_enum_values': 'old_multi_enum_values',
        'new_multi_enum_values': 'new_multi_enum_values',
        'new_approval_status': 'new_approval_status',
        'old_approval_status': 'old_approval_status',
        'duplicate_of': 'duplicate_of',
        'duplicated_from': 'duplicated_from',
        'dependency': 'dependency',
        'source': 'source',
        'target': 'target'
    }

    def __init__(self, gid=None, resource_type=None, created_at=None, resource_subtype=None, text=None, html_text=None, is_pinned=None, sticker_name=None, created_by=None, type=None, is_editable=None, is_edited=None, hearted=None, hearts=None, num_hearts=None, liked=None, likes=None, num_likes=None, previews=None, old_name=None, new_name=None, old_dates=None, new_dates=None, old_resource_subtype=None, new_resource_subtype=None, story=None, assignee=None, follower=None, old_section=None, new_section=None, task=None, project=None, tag=None, custom_field=None, old_text_value=None, new_text_value=None, old_number_value=None, new_number_value=None, old_enum_value=None, new_enum_value=None, old_date_value=None, new_date_value=None, old_people_value=None, new_people_value=None, old_multi_enum_values=None, new_multi_enum_values=None, new_approval_status=None, old_approval_status=None, duplicate_of=None, duplicated_from=None, dependency=None, source=None, target=None):  # noqa: E501
        """StoryResponse - a model defined in Swagger"""  # noqa: E501
        self._gid = None
        self._resource_type = None
        self._created_at = None
        self._resource_subtype = None
        self._text = None
        self._html_text = None
        self._is_pinned = None
        self._sticker_name = None
        self._created_by = None
        self._type = None
        self._is_editable = None
        self._is_edited = None
        self._hearted = None
        self._hearts = None
        self._num_hearts = None
        self._liked = None
        self._likes = None
        self._num_likes = None
        self._previews = None
        self._old_name = None
        self._new_name = None
        self._old_dates = None
        self._new_dates = None
        self._old_resource_subtype = None
        self._new_resource_subtype = None
        self._story = None
        self._assignee = None
        self._follower = None
        self._old_section = None
        self._new_section = None
        self._task = None
        self._project = None
        self._tag = None
        self._custom_field = None
        self._old_text_value = None
        self._new_text_value = None
        self._old_number_value = None
        self._new_number_value = None
        self._old_enum_value = None
        self._new_enum_value = None
        self._old_date_value = None
        self._new_date_value = None
        self._old_people_value = None
        self._new_people_value = None
        self._old_multi_enum_values = None
        self._new_multi_enum_values = None
        self._new_approval_status = None
        self._old_approval_status = None
        self._duplicate_of = None
        self._duplicated_from = None
        self._dependency = None
        self._source = None
        self._target = None
        self.discriminator = None
        if gid is not None:
            self.gid = gid
        if resource_type is not None:
            self.resource_type = resource_type
        if created_at is not None:
            self.created_at = created_at
        if resource_subtype is not None:
            self.resource_subtype = resource_subtype
        if text is not None:
            self.text = text
        if html_text is not None:
            self.html_text = html_text
        if is_pinned is not None:
            self.is_pinned = is_pinned
        if sticker_name is not None:
            self.sticker_name = sticker_name
        if created_by is not None:
            self.created_by = created_by
        if type is not None:
            self.type = type
        if is_editable is not None:
            self.is_editable = is_editable
        if is_edited is not None:
            self.is_edited = is_edited
        if hearted is not None:
            self.hearted = hearted
        if hearts is not None:
            self.hearts = hearts
        if num_hearts is not None:
            self.num_hearts = num_hearts
        if liked is not None:
            self.liked = liked
        if likes is not None:
            self.likes = likes
        if num_likes is not None:
            self.num_likes = num_likes
        if previews is not None:
            self.previews = previews
        if old_name is not None:
            self.old_name = old_name
        if new_name is not None:
            self.new_name = new_name
        if old_dates is not None:
            self.old_dates = old_dates
        if new_dates is not None:
            self.new_dates = new_dates
        if old_resource_subtype is not None:
            self.old_resource_subtype = old_resource_subtype
        if new_resource_subtype is not None:
            self.new_resource_subtype = new_resource_subtype
        if story is not None:
            self.story = story
        if assignee is not None:
            self.assignee = assignee
        if follower is not None:
            self.follower = follower
        if old_section is not None:
            self.old_section = old_section
        if new_section is not None:
            self.new_section = new_section
        if task is not None:
            self.task = task
        if project is not None:
            self.project = project
        if tag is not None:
            self.tag = tag
        if custom_field is not None:
            self.custom_field = custom_field
        if old_text_value is not None:
            self.old_text_value = old_text_value
        if new_text_value is not None:
            self.new_text_value = new_text_value
        if old_number_value is not None:
            self.old_number_value = old_number_value
        if new_number_value is not None:
            self.new_number_value = new_number_value
        if old_enum_value is not None:
            self.old_enum_value = old_enum_value
        if new_enum_value is not None:
            self.new_enum_value = new_enum_value
        if old_date_value is not None:
            self.old_date_value = old_date_value
        if new_date_value is not None:
            self.new_date_value = new_date_value
        if old_people_value is not None:
            self.old_people_value = old_people_value
        if new_people_value is not None:
            self.new_people_value = new_people_value
        if old_multi_enum_values is not None:
            self.old_multi_enum_values = old_multi_enum_values
        if new_multi_enum_values is not None:
            self.new_multi_enum_values = new_multi_enum_values
        if new_approval_status is not None:
            self.new_approval_status = new_approval_status
        if old_approval_status is not None:
            self.old_approval_status = old_approval_status
        if duplicate_of is not None:
            self.duplicate_of = duplicate_of
        if duplicated_from is not None:
            self.duplicated_from = duplicated_from
        if dependency is not None:
            self.dependency = dependency
        if source is not None:
            self.source = source
        if target is not None:
            self.target = target

    @property
    def gid(self):
        """Gets the gid of this StoryResponse.  # noqa: E501

        Globally unique identifier of the resource, as a string.  # noqa: E501

        :return: The gid of this StoryResponse.  # noqa: E501
        :rtype: str
        """
        return self._gid

    @gid.setter
    def gid(self, gid):
        """Sets the gid of this StoryResponse.

        Globally unique identifier of the resource, as a string.  # noqa: E501

        :param gid: The gid of this StoryResponse.  # noqa: E501
        :type: str
        """

        self._gid = gid

    @property
    def resource_type(self):
        """Gets the resource_type of this StoryResponse.  # noqa: E501

        The base type of this resource.  # noqa: E501

        :return: The resource_type of this StoryResponse.  # noqa: E501
        :rtype: str
        """
        return self._resource_type

    @resource_type.setter
    def resource_type(self, resource_type):
        """Sets the resource_type of this StoryResponse.

        The base type of this resource.  # noqa: E501

        :param resource_type: The resource_type of this StoryResponse.  # noqa: E501
        :type: str
        """

        self._resource_type = resource_type

    @property
    def created_at(self):
        """Gets the created_at of this StoryResponse.  # noqa: E501

        The time at which this resource was created.  # noqa: E501

        :return: The created_at of this StoryResponse.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this StoryResponse.

        The time at which this resource was created.  # noqa: E501

        :param created_at: The created_at of this StoryResponse.  # noqa: E501
        :type: datetime
        """

        self._created_at = created_at

    @property
    def resource_subtype(self):
        """Gets the resource_subtype of this StoryResponse.  # noqa: E501

        The subtype of this resource. Different subtypes retain many of the same fields and behavior, but may render differently in Asana or represent resources with different semantic meaning.  # noqa: E501

        :return: The resource_subtype of this StoryResponse.  # noqa: E501
        :rtype: str
        """
        return self._resource_subtype

    @resource_subtype.setter
    def resource_subtype(self, resource_subtype):
        """Sets the resource_subtype of this StoryResponse.

        The subtype of this resource. Different subtypes retain many of the same fields and behavior, but may render differently in Asana or represent resources with different semantic meaning.  # noqa: E501

        :param resource_subtype: The resource_subtype of this StoryResponse.  # noqa: E501
        :type: str
        """

        self._resource_subtype = resource_subtype

    @property
    def text(self):
        """Gets the text of this StoryResponse.  # noqa: E501

        The plain text of the comment to add. Cannot be used with html_text.  # noqa: E501

        :return: The text of this StoryResponse.  # noqa: E501
        :rtype: str
        """
        return self._text

    @text.setter
    def text(self, text):
        """Sets the text of this StoryResponse.

        The plain text of the comment to add. Cannot be used with html_text.  # noqa: E501

        :param text: The text of this StoryResponse.  # noqa: E501
        :type: str
        """

        self._text = text

    @property
    def html_text(self):
        """Gets the html_text of this StoryResponse.  # noqa: E501

        [Opt In](/docs/inputoutput-options). HTML formatted text for a comment. This will not include the name of the creator.  # noqa: E501

        :return: The html_text of this StoryResponse.  # noqa: E501
        :rtype: str
        """
        return self._html_text

    @html_text.setter
    def html_text(self, html_text):
        """Sets the html_text of this StoryResponse.

        [Opt In](/docs/inputoutput-options). HTML formatted text for a comment. This will not include the name of the creator.  # noqa: E501

        :param html_text: The html_text of this StoryResponse.  # noqa: E501
        :type: str
        """

        self._html_text = html_text

    @property
    def is_pinned(self):
        """Gets the is_pinned of this StoryResponse.  # noqa: E501

        *Conditional*. Whether the story should be pinned on the resource.  # noqa: E501

        :return: The is_pinned of this StoryResponse.  # noqa: E501
        :rtype: bool
        """
        return self._is_pinned

    @is_pinned.setter
    def is_pinned(self, is_pinned):
        """Sets the is_pinned of this StoryResponse.

        *Conditional*. Whether the story should be pinned on the resource.  # noqa: E501

        :param is_pinned: The is_pinned of this StoryResponse.  # noqa: E501
        :type: bool
        """

        self._is_pinned = is_pinned

    @property
    def sticker_name(self):
        """Gets the sticker_name of this StoryResponse.  # noqa: E501

        The name of the sticker in this story. `null` if there is no sticker.  # noqa: E501

        :return: The sticker_name of this StoryResponse.  # noqa: E501
        :rtype: str
        """
        return self._sticker_name

    @sticker_name.setter
    def sticker_name(self, sticker_name):
        """Sets the sticker_name of this StoryResponse.

        The name of the sticker in this story. `null` if there is no sticker.  # noqa: E501

        :param sticker_name: The sticker_name of this StoryResponse.  # noqa: E501
        :type: str
        """
        allowed_values = ["green_checkmark", "people_dancing", "dancing_unicorn", "heart", "party_popper", "people_waving_flags", "splashing_narwhal", "trophy", "yeti_riding_unicorn", "celebrating_people", "determined_climbers", "phoenix_spreading_love"]  # noqa: E501
        if sticker_name not in allowed_values:
            raise ValueError(
                "Invalid value for `sticker_name` ({0}), must be one of {1}"  # noqa: E501
                .format(sticker_name, allowed_values)
            )

        self._sticker_name = sticker_name

    @property
    def created_by(self):
        """Gets the created_by of this StoryResponse.  # noqa: E501


        :return: The created_by of this StoryResponse.  # noqa: E501
        :rtype: CustomFieldResponsePeopleValue
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """Sets the created_by of this StoryResponse.


        :param created_by: The created_by of this StoryResponse.  # noqa: E501
        :type: CustomFieldResponsePeopleValue
        """

        self._created_by = created_by

    @property
    def type(self):
        """Gets the type of this StoryResponse.  # noqa: E501


        :return: The type of this StoryResponse.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this StoryResponse.


        :param type: The type of this StoryResponse.  # noqa: E501
        :type: str
        """
        allowed_values = ["comment", "system"]  # noqa: E501
        if type not in allowed_values:
            raise ValueError(
                "Invalid value for `type` ({0}), must be one of {1}"  # noqa: E501
                .format(type, allowed_values)
            )

        self._type = type

    @property
    def is_editable(self):
        """Gets the is_editable of this StoryResponse.  # noqa: E501

        *Conditional*. Whether the text of the story can be edited after creation.  # noqa: E501

        :return: The is_editable of this StoryResponse.  # noqa: E501
        :rtype: bool
        """
        return self._is_editable

    @is_editable.setter
    def is_editable(self, is_editable):
        """Sets the is_editable of this StoryResponse.

        *Conditional*. Whether the text of the story can be edited after creation.  # noqa: E501

        :param is_editable: The is_editable of this StoryResponse.  # noqa: E501
        :type: bool
        """

        self._is_editable = is_editable

    @property
    def is_edited(self):
        """Gets the is_edited of this StoryResponse.  # noqa: E501

        *Conditional*. Whether the text of the story has been edited after creation.  # noqa: E501

        :return: The is_edited of this StoryResponse.  # noqa: E501
        :rtype: bool
        """
        return self._is_edited

    @is_edited.setter
    def is_edited(self, is_edited):
        """Sets the is_edited of this StoryResponse.

        *Conditional*. Whether the text of the story has been edited after creation.  # noqa: E501

        :param is_edited: The is_edited of this StoryResponse.  # noqa: E501
        :type: bool
        """

        self._is_edited = is_edited

    @property
    def hearted(self):
        """Gets the hearted of this StoryResponse.  # noqa: E501

        *Deprecated - please use likes instead* *Conditional*. True if the story is hearted by the authorized user, false if not.  # noqa: E501

        :return: The hearted of this StoryResponse.  # noqa: E501
        :rtype: bool
        """
        return self._hearted

    @hearted.setter
    def hearted(self, hearted):
        """Sets the hearted of this StoryResponse.

        *Deprecated - please use likes instead* *Conditional*. True if the story is hearted by the authorized user, false if not.  # noqa: E501

        :param hearted: The hearted of this StoryResponse.  # noqa: E501
        :type: bool
        """

        self._hearted = hearted

    @property
    def hearts(self):
        """Gets the hearts of this StoryResponse.  # noqa: E501

        *Deprecated - please use likes instead*  *Conditional*. Array of likes for users who have hearted this story.  # noqa: E501

        :return: The hearts of this StoryResponse.  # noqa: E501
        :rtype: list[GoalResponseLikes]
        """
        return self._hearts

    @hearts.setter
    def hearts(self, hearts):
        """Sets the hearts of this StoryResponse.

        *Deprecated - please use likes instead*  *Conditional*. Array of likes for users who have hearted this story.  # noqa: E501

        :param hearts: The hearts of this StoryResponse.  # noqa: E501
        :type: list[GoalResponseLikes]
        """

        self._hearts = hearts

    @property
    def num_hearts(self):
        """Gets the num_hearts of this StoryResponse.  # noqa: E501

        *Deprecated - please use likes instead*  *Conditional*. The number of users who have hearted this story.  # noqa: E501

        :return: The num_hearts of this StoryResponse.  # noqa: E501
        :rtype: int
        """
        return self._num_hearts

    @num_hearts.setter
    def num_hearts(self, num_hearts):
        """Sets the num_hearts of this StoryResponse.

        *Deprecated - please use likes instead*  *Conditional*. The number of users who have hearted this story.  # noqa: E501

        :param num_hearts: The num_hearts of this StoryResponse.  # noqa: E501
        :type: int
        """

        self._num_hearts = num_hearts

    @property
    def liked(self):
        """Gets the liked of this StoryResponse.  # noqa: E501

        *Conditional*. True if the story is liked by the authorized user, false if not.  # noqa: E501

        :return: The liked of this StoryResponse.  # noqa: E501
        :rtype: bool
        """
        return self._liked

    @liked.setter
    def liked(self, liked):
        """Sets the liked of this StoryResponse.

        *Conditional*. True if the story is liked by the authorized user, false if not.  # noqa: E501

        :param liked: The liked of this StoryResponse.  # noqa: E501
        :type: bool
        """

        self._liked = liked

    @property
    def likes(self):
        """Gets the likes of this StoryResponse.  # noqa: E501

        *Conditional*. Array of likes for users who have liked this story.  # noqa: E501

        :return: The likes of this StoryResponse.  # noqa: E501
        :rtype: list[GoalResponseLikes]
        """
        return self._likes

    @likes.setter
    def likes(self, likes):
        """Sets the likes of this StoryResponse.

        *Conditional*. Array of likes for users who have liked this story.  # noqa: E501

        :param likes: The likes of this StoryResponse.  # noqa: E501
        :type: list[GoalResponseLikes]
        """

        self._likes = likes

    @property
    def num_likes(self):
        """Gets the num_likes of this StoryResponse.  # noqa: E501

        *Conditional*. The number of users who have liked this story.  # noqa: E501

        :return: The num_likes of this StoryResponse.  # noqa: E501
        :rtype: int
        """
        return self._num_likes

    @num_likes.setter
    def num_likes(self, num_likes):
        """Sets the num_likes of this StoryResponse.

        *Conditional*. The number of users who have liked this story.  # noqa: E501

        :param num_likes: The num_likes of this StoryResponse.  # noqa: E501
        :type: int
        """

        self._num_likes = num_likes

    @property
    def previews(self):
        """Gets the previews of this StoryResponse.  # noqa: E501

        *Conditional*. A collection of previews to be displayed in the story.  *Note: This property only exists for comment stories.*  # noqa: E501

        :return: The previews of this StoryResponse.  # noqa: E501
        :rtype: list[StoryResponsePreviews]
        """
        return self._previews

    @previews.setter
    def previews(self, previews):
        """Sets the previews of this StoryResponse.

        *Conditional*. A collection of previews to be displayed in the story.  *Note: This property only exists for comment stories.*  # noqa: E501

        :param previews: The previews of this StoryResponse.  # noqa: E501
        :type: list[StoryResponsePreviews]
        """

        self._previews = previews

    @property
    def old_name(self):
        """Gets the old_name of this StoryResponse.  # noqa: E501

        *Conditional*'  # noqa: E501

        :return: The old_name of this StoryResponse.  # noqa: E501
        :rtype: str
        """
        return self._old_name

    @old_name.setter
    def old_name(self, old_name):
        """Sets the old_name of this StoryResponse.

        *Conditional*'  # noqa: E501

        :param old_name: The old_name of this StoryResponse.  # noqa: E501
        :type: str
        """

        self._old_name = old_name

    @property
    def new_name(self):
        """Gets the new_name of this StoryResponse.  # noqa: E501

        *Conditional*  # noqa: E501

        :return: The new_name of this StoryResponse.  # noqa: E501
        :rtype: str
        """
        return self._new_name

    @new_name.setter
    def new_name(self, new_name):
        """Sets the new_name of this StoryResponse.

        *Conditional*  # noqa: E501

        :param new_name: The new_name of this StoryResponse.  # noqa: E501
        :type: str
        """

        self._new_name = new_name

    @property
    def old_dates(self):
        """Gets the old_dates of this StoryResponse.  # noqa: E501


        :return: The old_dates of this StoryResponse.  # noqa: E501
        :rtype: StoryResponseOldDates
        """
        return self._old_dates

    @old_dates.setter
    def old_dates(self, old_dates):
        """Sets the old_dates of this StoryResponse.


        :param old_dates: The old_dates of this StoryResponse.  # noqa: E501
        :type: StoryResponseOldDates
        """

        self._old_dates = old_dates

    @property
    def new_dates(self):
        """Gets the new_dates of this StoryResponse.  # noqa: E501


        :return: The new_dates of this StoryResponse.  # noqa: E501
        :rtype: StoryResponseOldDates
        """
        return self._new_dates

    @new_dates.setter
    def new_dates(self, new_dates):
        """Sets the new_dates of this StoryResponse.


        :param new_dates: The new_dates of this StoryResponse.  # noqa: E501
        :type: StoryResponseOldDates
        """

        self._new_dates = new_dates

    @property
    def old_resource_subtype(self):
        """Gets the old_resource_subtype of this StoryResponse.  # noqa: E501

        *Conditional*  # noqa: E501

        :return: The old_resource_subtype of this StoryResponse.  # noqa: E501
        :rtype: str
        """
        return self._old_resource_subtype

    @old_resource_subtype.setter
    def old_resource_subtype(self, old_resource_subtype):
        """Sets the old_resource_subtype of this StoryResponse.

        *Conditional*  # noqa: E501

        :param old_resource_subtype: The old_resource_subtype of this StoryResponse.  # noqa: E501
        :type: str
        """

        self._old_resource_subtype = old_resource_subtype

    @property
    def new_resource_subtype(self):
        """Gets the new_resource_subtype of this StoryResponse.  # noqa: E501

        *Conditional*  # noqa: E501

        :return: The new_resource_subtype of this StoryResponse.  # noqa: E501
        :rtype: str
        """
        return self._new_resource_subtype

    @new_resource_subtype.setter
    def new_resource_subtype(self, new_resource_subtype):
        """Sets the new_resource_subtype of this StoryResponse.

        *Conditional*  # noqa: E501

        :param new_resource_subtype: The new_resource_subtype of this StoryResponse.  # noqa: E501
        :type: str
        """

        self._new_resource_subtype = new_resource_subtype

    @property
    def story(self):
        """Gets the story of this StoryResponse.  # noqa: E501


        :return: The story of this StoryResponse.  # noqa: E501
        :rtype: StoryResponseStory
        """
        return self._story

    @story.setter
    def story(self, story):
        """Sets the story of this StoryResponse.


        :param story: The story of this StoryResponse.  # noqa: E501
        :type: StoryResponseStory
        """

        self._story = story

    @property
    def assignee(self):
        """Gets the assignee of this StoryResponse.  # noqa: E501


        :return: The assignee of this StoryResponse.  # noqa: E501
        :rtype: StoryResponseAssignee
        """
        return self._assignee

    @assignee.setter
    def assignee(self, assignee):
        """Sets the assignee of this StoryResponse.


        :param assignee: The assignee of this StoryResponse.  # noqa: E501
        :type: StoryResponseAssignee
        """

        self._assignee = assignee

    @property
    def follower(self):
        """Gets the follower of this StoryResponse.  # noqa: E501


        :return: The follower of this StoryResponse.  # noqa: E501
        :rtype: StoryResponseAssignee
        """
        return self._follower

    @follower.setter
    def follower(self, follower):
        """Sets the follower of this StoryResponse.


        :param follower: The follower of this StoryResponse.  # noqa: E501
        :type: StoryResponseAssignee
        """

        self._follower = follower

    @property
    def old_section(self):
        """Gets the old_section of this StoryResponse.  # noqa: E501


        :return: The old_section of this StoryResponse.  # noqa: E501
        :rtype: StoryResponseOldSection
        """
        return self._old_section

    @old_section.setter
    def old_section(self, old_section):
        """Sets the old_section of this StoryResponse.


        :param old_section: The old_section of this StoryResponse.  # noqa: E501
        :type: StoryResponseOldSection
        """

        self._old_section = old_section

    @property
    def new_section(self):
        """Gets the new_section of this StoryResponse.  # noqa: E501


        :return: The new_section of this StoryResponse.  # noqa: E501
        :rtype: StoryResponseOldSection
        """
        return self._new_section

    @new_section.setter
    def new_section(self, new_section):
        """Sets the new_section of this StoryResponse.


        :param new_section: The new_section of this StoryResponse.  # noqa: E501
        :type: StoryResponseOldSection
        """

        self._new_section = new_section

    @property
    def task(self):
        """Gets the task of this StoryResponse.  # noqa: E501


        :return: The task of this StoryResponse.  # noqa: E501
        :rtype: StoryResponseTask
        """
        return self._task

    @task.setter
    def task(self, task):
        """Sets the task of this StoryResponse.


        :param task: The task of this StoryResponse.  # noqa: E501
        :type: StoryResponseTask
        """

        self._task = task

    @property
    def project(self):
        """Gets the project of this StoryResponse.  # noqa: E501


        :return: The project of this StoryResponse.  # noqa: E501
        :rtype: StoryResponseProject
        """
        return self._project

    @project.setter
    def project(self, project):
        """Sets the project of this StoryResponse.


        :param project: The project of this StoryResponse.  # noqa: E501
        :type: StoryResponseProject
        """

        self._project = project

    @property
    def tag(self):
        """Gets the tag of this StoryResponse.  # noqa: E501


        :return: The tag of this StoryResponse.  # noqa: E501
        :rtype: StoryResponseTag
        """
        return self._tag

    @tag.setter
    def tag(self, tag):
        """Sets the tag of this StoryResponse.


        :param tag: The tag of this StoryResponse.  # noqa: E501
        :type: StoryResponseTag
        """

        self._tag = tag

    @property
    def custom_field(self):
        """Gets the custom_field of this StoryResponse.  # noqa: E501


        :return: The custom_field of this StoryResponse.  # noqa: E501
        :rtype: StoryResponseCustomField
        """
        return self._custom_field

    @custom_field.setter
    def custom_field(self, custom_field):
        """Sets the custom_field of this StoryResponse.


        :param custom_field: The custom_field of this StoryResponse.  # noqa: E501
        :type: StoryResponseCustomField
        """

        self._custom_field = custom_field

    @property
    def old_text_value(self):
        """Gets the old_text_value of this StoryResponse.  # noqa: E501

        *Conditional*  # noqa: E501

        :return: The old_text_value of this StoryResponse.  # noqa: E501
        :rtype: str
        """
        return self._old_text_value

    @old_text_value.setter
    def old_text_value(self, old_text_value):
        """Sets the old_text_value of this StoryResponse.

        *Conditional*  # noqa: E501

        :param old_text_value: The old_text_value of this StoryResponse.  # noqa: E501
        :type: str
        """

        self._old_text_value = old_text_value

    @property
    def new_text_value(self):
        """Gets the new_text_value of this StoryResponse.  # noqa: E501

        *Conditional*  # noqa: E501

        :return: The new_text_value of this StoryResponse.  # noqa: E501
        :rtype: str
        """
        return self._new_text_value

    @new_text_value.setter
    def new_text_value(self, new_text_value):
        """Sets the new_text_value of this StoryResponse.

        *Conditional*  # noqa: E501

        :param new_text_value: The new_text_value of this StoryResponse.  # noqa: E501
        :type: str
        """

        self._new_text_value = new_text_value

    @property
    def old_number_value(self):
        """Gets the old_number_value of this StoryResponse.  # noqa: E501

        *Conditional*  # noqa: E501

        :return: The old_number_value of this StoryResponse.  # noqa: E501
        :rtype: int
        """
        return self._old_number_value

    @old_number_value.setter
    def old_number_value(self, old_number_value):
        """Sets the old_number_value of this StoryResponse.

        *Conditional*  # noqa: E501

        :param old_number_value: The old_number_value of this StoryResponse.  # noqa: E501
        :type: int
        """

        self._old_number_value = old_number_value

    @property
    def new_number_value(self):
        """Gets the new_number_value of this StoryResponse.  # noqa: E501

        *Conditional*  # noqa: E501

        :return: The new_number_value of this StoryResponse.  # noqa: E501
        :rtype: int
        """
        return self._new_number_value

    @new_number_value.setter
    def new_number_value(self, new_number_value):
        """Sets the new_number_value of this StoryResponse.

        *Conditional*  # noqa: E501

        :param new_number_value: The new_number_value of this StoryResponse.  # noqa: E501
        :type: int
        """

        self._new_number_value = new_number_value

    @property
    def old_enum_value(self):
        """Gets the old_enum_value of this StoryResponse.  # noqa: E501


        :return: The old_enum_value of this StoryResponse.  # noqa: E501
        :rtype: StoryResponseOldEnumValue
        """
        return self._old_enum_value

    @old_enum_value.setter
    def old_enum_value(self, old_enum_value):
        """Sets the old_enum_value of this StoryResponse.


        :param old_enum_value: The old_enum_value of this StoryResponse.  # noqa: E501
        :type: StoryResponseOldEnumValue
        """

        self._old_enum_value = old_enum_value

    @property
    def new_enum_value(self):
        """Gets the new_enum_value of this StoryResponse.  # noqa: E501


        :return: The new_enum_value of this StoryResponse.  # noqa: E501
        :rtype: StoryResponseOldEnumValue
        """
        return self._new_enum_value

    @new_enum_value.setter
    def new_enum_value(self, new_enum_value):
        """Sets the new_enum_value of this StoryResponse.


        :param new_enum_value: The new_enum_value of this StoryResponse.  # noqa: E501
        :type: StoryResponseOldEnumValue
        """

        self._new_enum_value = new_enum_value

    @property
    def old_date_value(self):
        """Gets the old_date_value of this StoryResponse.  # noqa: E501


        :return: The old_date_value of this StoryResponse.  # noqa: E501
        :rtype: AllOfStoryResponseOldDateValue
        """
        return self._old_date_value

    @old_date_value.setter
    def old_date_value(self, old_date_value):
        """Sets the old_date_value of this StoryResponse.


        :param old_date_value: The old_date_value of this StoryResponse.  # noqa: E501
        :type: AllOfStoryResponseOldDateValue
        """

        self._old_date_value = old_date_value

    @property
    def new_date_value(self):
        """Gets the new_date_value of this StoryResponse.  # noqa: E501


        :return: The new_date_value of this StoryResponse.  # noqa: E501
        :rtype: AllOfStoryResponseNewDateValue
        """
        return self._new_date_value

    @new_date_value.setter
    def new_date_value(self, new_date_value):
        """Sets the new_date_value of this StoryResponse.


        :param new_date_value: The new_date_value of this StoryResponse.  # noqa: E501
        :type: AllOfStoryResponseNewDateValue
        """

        self._new_date_value = new_date_value

    @property
    def old_people_value(self):
        """Gets the old_people_value of this StoryResponse.  # noqa: E501

        *Conditional*. The old value of a people custom field story.  # noqa: E501

        :return: The old_people_value of this StoryResponse.  # noqa: E501
        :rtype: list[CustomFieldResponsePeopleValue]
        """
        return self._old_people_value

    @old_people_value.setter
    def old_people_value(self, old_people_value):
        """Sets the old_people_value of this StoryResponse.

        *Conditional*. The old value of a people custom field story.  # noqa: E501

        :param old_people_value: The old_people_value of this StoryResponse.  # noqa: E501
        :type: list[CustomFieldResponsePeopleValue]
        """

        self._old_people_value = old_people_value

    @property
    def new_people_value(self):
        """Gets the new_people_value of this StoryResponse.  # noqa: E501

        *Conditional*. The new value of a people custom field story.  # noqa: E501

        :return: The new_people_value of this StoryResponse.  # noqa: E501
        :rtype: list[CustomFieldResponsePeopleValue]
        """
        return self._new_people_value

    @new_people_value.setter
    def new_people_value(self, new_people_value):
        """Sets the new_people_value of this StoryResponse.

        *Conditional*. The new value of a people custom field story.  # noqa: E501

        :param new_people_value: The new_people_value of this StoryResponse.  # noqa: E501
        :type: list[CustomFieldResponsePeopleValue]
        """

        self._new_people_value = new_people_value

    @property
    def old_multi_enum_values(self):
        """Gets the old_multi_enum_values of this StoryResponse.  # noqa: E501

        *Conditional*. The old value of a multi-enum custom field story.  # noqa: E501

        :return: The old_multi_enum_values of this StoryResponse.  # noqa: E501
        :rtype: list[CustomFieldBaseEnumOptions]
        """
        return self._old_multi_enum_values

    @old_multi_enum_values.setter
    def old_multi_enum_values(self, old_multi_enum_values):
        """Sets the old_multi_enum_values of this StoryResponse.

        *Conditional*. The old value of a multi-enum custom field story.  # noqa: E501

        :param old_multi_enum_values: The old_multi_enum_values of this StoryResponse.  # noqa: E501
        :type: list[CustomFieldBaseEnumOptions]
        """

        self._old_multi_enum_values = old_multi_enum_values

    @property
    def new_multi_enum_values(self):
        """Gets the new_multi_enum_values of this StoryResponse.  # noqa: E501

        *Conditional*. The new value of a multi-enum custom field story.  # noqa: E501

        :return: The new_multi_enum_values of this StoryResponse.  # noqa: E501
        :rtype: list[CustomFieldBaseEnumOptions]
        """
        return self._new_multi_enum_values

    @new_multi_enum_values.setter
    def new_multi_enum_values(self, new_multi_enum_values):
        """Sets the new_multi_enum_values of this StoryResponse.

        *Conditional*. The new value of a multi-enum custom field story.  # noqa: E501

        :param new_multi_enum_values: The new_multi_enum_values of this StoryResponse.  # noqa: E501
        :type: list[CustomFieldBaseEnumOptions]
        """

        self._new_multi_enum_values = new_multi_enum_values

    @property
    def new_approval_status(self):
        """Gets the new_approval_status of this StoryResponse.  # noqa: E501

        *Conditional*. The new value of approval status.  # noqa: E501

        :return: The new_approval_status of this StoryResponse.  # noqa: E501
        :rtype: str
        """
        return self._new_approval_status

    @new_approval_status.setter
    def new_approval_status(self, new_approval_status):
        """Sets the new_approval_status of this StoryResponse.

        *Conditional*. The new value of approval status.  # noqa: E501

        :param new_approval_status: The new_approval_status of this StoryResponse.  # noqa: E501
        :type: str
        """

        self._new_approval_status = new_approval_status

    @property
    def old_approval_status(self):
        """Gets the old_approval_status of this StoryResponse.  # noqa: E501

        *Conditional*. The old value of approval status.  # noqa: E501

        :return: The old_approval_status of this StoryResponse.  # noqa: E501
        :rtype: str
        """
        return self._old_approval_status

    @old_approval_status.setter
    def old_approval_status(self, old_approval_status):
        """Sets the old_approval_status of this StoryResponse.

        *Conditional*. The old value of approval status.  # noqa: E501

        :param old_approval_status: The old_approval_status of this StoryResponse.  # noqa: E501
        :type: str
        """

        self._old_approval_status = old_approval_status

    @property
    def duplicate_of(self):
        """Gets the duplicate_of of this StoryResponse.  # noqa: E501


        :return: The duplicate_of of this StoryResponse.  # noqa: E501
        :rtype: StoryResponseTask
        """
        return self._duplicate_of

    @duplicate_of.setter
    def duplicate_of(self, duplicate_of):
        """Sets the duplicate_of of this StoryResponse.


        :param duplicate_of: The duplicate_of of this StoryResponse.  # noqa: E501
        :type: StoryResponseTask
        """

        self._duplicate_of = duplicate_of

    @property
    def duplicated_from(self):
        """Gets the duplicated_from of this StoryResponse.  # noqa: E501


        :return: The duplicated_from of this StoryResponse.  # noqa: E501
        :rtype: StoryResponseTask
        """
        return self._duplicated_from

    @duplicated_from.setter
    def duplicated_from(self, duplicated_from):
        """Sets the duplicated_from of this StoryResponse.


        :param duplicated_from: The duplicated_from of this StoryResponse.  # noqa: E501
        :type: StoryResponseTask
        """

        self._duplicated_from = duplicated_from

    @property
    def dependency(self):
        """Gets the dependency of this StoryResponse.  # noqa: E501


        :return: The dependency of this StoryResponse.  # noqa: E501
        :rtype: StoryResponseTask
        """
        return self._dependency

    @dependency.setter
    def dependency(self, dependency):
        """Sets the dependency of this StoryResponse.


        :param dependency: The dependency of this StoryResponse.  # noqa: E501
        :type: StoryResponseTask
        """

        self._dependency = dependency

    @property
    def source(self):
        """Gets the source of this StoryResponse.  # noqa: E501

        The component of the Asana product the user used to trigger the story.  # noqa: E501

        :return: The source of this StoryResponse.  # noqa: E501
        :rtype: str
        """
        return self._source

    @source.setter
    def source(self, source):
        """Sets the source of this StoryResponse.

        The component of the Asana product the user used to trigger the story.  # noqa: E501

        :param source: The source of this StoryResponse.  # noqa: E501
        :type: str
        """
        allowed_values = ["web", "email", "mobile", "api", "unknown"]  # noqa: E501
        if source not in allowed_values:
            raise ValueError(
                "Invalid value for `source` ({0}), must be one of {1}"  # noqa: E501
                .format(source, allowed_values)
            )

        self._source = source

    @property
    def target(self):
        """Gets the target of this StoryResponse.  # noqa: E501


        :return: The target of this StoryResponse.  # noqa: E501
        :rtype: StoryResponseTarget
        """
        return self._target

    @target.setter
    def target(self, target):
        """Sets the target of this StoryResponse.


        :param target: The target of this StoryResponse.  # noqa: E501
        :type: StoryResponseTarget
        """

        self._target = target

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(StoryResponse, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, StoryResponse):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Asana

    This is the interface for interacting with the [Asana Platform](https://developers.asana.com). Our API reference is generated from our [OpenAPI spec] (https://raw.githubusercontent.com/Asana/openapi/master/defs/asana_oas.yaml).  # noqa: E501

    OpenAPI spec version: 1.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class BatchResponse(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'status_code': 'int',
        'headers': 'object',
        'body': 'object'
    }

    attribute_map = {
        'status_code': 'status_code',
        'headers': 'headers',
        'body': 'body'
    }

    def __init__(self, status_code=None, headers=None, body=None):  # noqa: E501
        """BatchResponse - a model defined in Swagger"""  # noqa: E501
        self._status_code = None
        self._headers = None
        self._body = None
        self.discriminator = None
        if status_code is not None:
            self.status_code = status_code
        if headers is not None:
            self.headers = headers
        if body is not None:
            self.body = body

    @property
    def status_code(self):
        """Gets the status_code of this BatchResponse.  # noqa: E501

        The HTTP status code that the invoked endpoint returned.  # noqa: E501

        :return: The status_code of this BatchResponse.  # noqa: E501
        :rtype: int
        """
        return self._status_code

    @status_code.setter
    def status_code(self, status_code):
        """Sets the status_code of this BatchResponse.

        The HTTP status code that the invoked endpoint returned.  # noqa: E501

        :param status_code: The status_code of this BatchResponse.  # noqa: E501
        :type: int
        """

        self._status_code = status_code

    @property
    def headers(self):
        """Gets the headers of this BatchResponse.  # noqa: E501

        A map of HTTP headers specific to this result. This is primarily used for returning a `Location` header to accompany a `201 Created` result.  The parent HTTP response will contain all common headers.  # noqa: E501

        :return: The headers of this BatchResponse.  # noqa: E501
        :rtype: object
        """
        return self._headers

    @headers.setter
    def headers(self, headers):
        """Sets the headers of this BatchResponse.

        A map of HTTP headers specific to this result. This is primarily used for returning a `Location` header to accompany a `201 Created` result.  The parent HTTP response will contain all common headers.  # noqa: E501

        :param headers: The headers of this BatchResponse.  # noqa: E501
        :type: object
        """

        self._headers = headers

    @property
    def body(self):
        """Gets the body of this BatchResponse.  # noqa: E501

        The JSON body that the invoked endpoint returned.  # noqa: E501

        :return: The body of this BatchResponse.  # noqa: E501
        :rtype: object
        """
        return self._body

    @body.setter
    def body(self, body):
        """Sets the body of this BatchResponse.

        The JSON body that the invoked endpoint returned.  # noqa: E501

        :param body: The body of this BatchResponse.  # noqa: E501
        :type: object
        """

        self._body = body

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(BatchResponse, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BatchResponse):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Asana

    This is the interface for interacting with the [Asana Platform](https://developers.asana.com). Our API reference is generated from our [OpenAPI spec] (https://raw.githubusercontent.com/Asana/openapi/master/defs/asana_oas.yaml).  # noqa: E501

    OpenAPI spec version: 1.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import unittest

import asana
from asana.api.projects_api import ProjectsApi  # noqa: E501
from asana.rest import ApiException


class TestProjectsApi(unittest.TestCase):
    """ProjectsApi unit test stubs"""

    def setUp(self):
        self.api = ProjectsApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_add_custom_field_setting_for_project(self):
        """Test case for add_custom_field_setting_for_project

        Add a custom field to a project  # noqa: E501
        """
        pass

    def test_add_followers_for_project(self):
        """Test case for add_followers_for_project

        Add followers to a project  # noqa: E501
        """
        pass

    def test_add_members_for_project(self):
        """Test case for add_members_for_project

        Add users to a project  # noqa: E501
        """
        pass

    def test_create_project(self):
        """Test case for create_project

        Create a project  # noqa: E501
        """
        pass

    def test_create_project_for_team(self):
        """Test case for create_project_for_team

        Create a project in a team  # noqa: E501
        """
        pass

    def test_create_project_for_workspace(self):
        """Test case for create_project_for_workspace

        Create a project in a workspace  # noqa: E501
        """
        pass

    def test_delete_project(self):
        """Test case for delete_project

        Delete a project  # noqa: E501
        """
        pass

    def test_duplicate_project(self):
        """Test case for duplicate_project

        Duplicate a project  # noqa: E501
        """
        pass

    def test_get_project(self):
        """Test case for get_project

        Get a project  # noqa: E501
        """
        pass

    def test_get_projects(self):
        """Test case for get_projects

        Get multiple projects  # noqa: E501
        """
        pass

    def test_get_projects_for_task(self):
        """Test case for get_projects_for_task

        Get projects a task is in  # noqa: E501
        """
        pass

    def test_get_projects_for_team(self):
        """Test case for get_projects_for_team

        Get a team's projects  # noqa: E501
        """
        pass

    def test_get_projects_for_workspace(self):
        """Test case for get_projects_for_workspace

        Get all projects in a workspace  # noqa: E501
        """
        pass

    def test_get_task_counts_for_project(self):
        """Test case for get_task_counts_for_project

        Get task count of a project  # noqa: E501
        """
        pass

    def test_project_save_as_template(self):
        """Test case for project_save_as_template

        Create a project template from a project  # noqa: E501
        """
        pass

    def test_remove_custom_field_setting_for_project(self):
        """Test case for remove_custom_field_setting_for_project

        Remove a custom field from a project  # noqa: E501
        """
        pass

    def test_remove_followers_for_project(self):
        """Test case for remove_followers_for_project

        Remove followers from a project  # noqa: E501
        """
        pass

    def test_remove_members_for_project(self):
        """Test case for remove_members_for_project

        Remove users from a project  # noqa: E501
        """
        pass

    def test_update_project(self):
        """Test case for update_project

        Update a project  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Asana

    This is the interface for interacting with the [Asana Platform](https://developers.asana.com). Our API reference is generated from our [OpenAPI spec] (https://raw.githubusercontent.com/Asana/openapi/master/defs/asana_oas.yaml).  # noqa: E501

    OpenAPI spec version: 1.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from asana.api_client import ApiClient


class ProjectsApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def add_custom_field_setting_for_project(self, body, project_gid, **kwargs):  # noqa: E501
        """Add a custom field to a project  # noqa: E501

        Custom fields are associated with projects by way of custom field settings.  This method creates a setting for the project.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_custom_field_setting_for_project(body, project_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProjectGidAddCustomFieldSettingBody body: Information about the custom field setting. (required)
        :param str project_gid: Globally unique identifier for the project. (required)
        :return: CustomFieldSettingResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = kwargs.get("_return_http_data_only", True)
        if kwargs.get('async_req'):
            return self.add_custom_field_setting_for_project_with_http_info(body, project_gid, **kwargs)  # noqa: E501
        else:
            (data) = self.add_custom_field_setting_for_project_with_http_info(body, project_gid, **kwargs)  # noqa: E501
            return data

    def add_custom_field_setting_for_project_with_http_info(self, body, project_gid, **kwargs):  # noqa: E501
        """Add a custom field to a project  # noqa: E501

        Custom fields are associated with projects by way of custom field settings.  This method creates a setting for the project.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_custom_field_setting_for_project_with_http_info(body, project_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProjectGidAddCustomFieldSettingBody body: Information about the custom field setting. (required)
        :param str project_gid: Globally unique identifier for the project. (required)
        :return: CustomFieldSettingResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'project_gid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('header_params')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method add_custom_field_setting_for_project" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `add_custom_field_setting_for_project`")  # noqa: E501
        # verify the required parameter 'project_gid' is set
        if ('project_gid' not in params or
                params['project_gid'] is None):
            raise ValueError("Missing the required parameter `project_gid` when calling `add_custom_field_setting_for_project`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'project_gid' in params:
            path_params['project_gid'] = params['project_gid']  # noqa: E501

        query_params = []

        header_params = kwargs.get("header_params", {})

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json; charset=UTF-8'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json; charset=UTF-8'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oauth2']  # noqa: E501

        return self.api_client.call_api(
            '/projects/{project_gid}/addCustomFieldSetting', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='CustomFieldSettingResponseData',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def add_followers_for_project(self, body, project_gid, **kwargs):  # noqa: E501
        """Add followers to a project  # noqa: E501

        Adds the specified list of users as followers to the project. Followers are a subset of members who have opted in to receive \"tasks added\" notifications for a project. Therefore, if the users are not already members of the project, they will also become members as a result of this operation. Returns the updated project record.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_followers_for_project(body, project_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProjectGidAddFollowersBody body: Information about the followers being added. (required)
        :param str project_gid: Globally unique identifier for the project. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: ProjectResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = kwargs.get("_return_http_data_only", True)
        if kwargs.get('async_req'):
            return self.add_followers_for_project_with_http_info(body, project_gid, **kwargs)  # noqa: E501
        else:
            (data) = self.add_followers_for_project_with_http_info(body, project_gid, **kwargs)  # noqa: E501
            return data

    def add_followers_for_project_with_http_info(self, body, project_gid, **kwargs):  # noqa: E501
        """Add followers to a project  # noqa: E501

        Adds the specified list of users as followers to the project. Followers are a subset of members who have opted in to receive \"tasks added\" notifications for a project. Therefore, if the users are not already members of the project, they will also become members as a result of this operation. Returns the updated project record.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_followers_for_project_with_http_info(body, project_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProjectGidAddFollowersBody body: Information about the followers being added. (required)
        :param str project_gid: Globally unique identifier for the project. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: ProjectResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'project_gid', 'opt_fields']  # noqa: E501
        all_params.append('async_req')
        all_params.append('header_params')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method add_followers_for_project" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `add_followers_for_project`")  # noqa: E501
        # verify the required parameter 'project_gid' is set
        if ('project_gid' not in params or
                params['project_gid'] is None):
            raise ValueError("Missing the required parameter `project_gid` when calling `add_followers_for_project`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'project_gid' in params:
            path_params['project_gid'] = params['project_gid']  # noqa: E501

        query_params = []
        if 'opt_fields' in params:
            query_params.append(('opt_fields', params['opt_fields']))  # noqa: E501
            collection_formats['opt_fields'] = 'csv'  # noqa: E501

        header_params = kwargs.get("header_params", {})

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json; charset=UTF-8'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json; charset=UTF-8'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oauth2']  # noqa: E501

        return self.api_client.call_api(
            '/projects/{project_gid}/addFollowers', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='ProjectResponseData',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def add_members_for_project(self, body, project_gid, **kwargs):  # noqa: E501
        """Add users to a project  # noqa: E501

        Adds the specified list of users as members of the project. Note that a user being added as a member may also be added as a *follower* as a result of this operation. This is because the user's default notification settings (i.e., in the \"Notifcations\" tab of \"My Profile Settings\") will override this endpoint's default behavior of setting \"Tasks added\" notifications to `false`. Returns the updated project record.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_members_for_project(body, project_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProjectGidAddMembersBody body: Information about the members being added. (required)
        :param str project_gid: Globally unique identifier for the project. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: ProjectResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = kwargs.get("_return_http_data_only", True)
        if kwargs.get('async_req'):
            return self.add_members_for_project_with_http_info(body, project_gid, **kwargs)  # noqa: E501
        else:
            (data) = self.add_members_for_project_with_http_info(body, project_gid, **kwargs)  # noqa: E501
            return data

    def add_members_for_project_with_http_info(self, body, project_gid, **kwargs):  # noqa: E501
        """Add users to a project  # noqa: E501

        Adds the specified list of users as members of the project. Note that a user being added as a member may also be added as a *follower* as a result of this operation. This is because the user's default notification settings (i.e., in the \"Notifcations\" tab of \"My Profile Settings\") will override this endpoint's default behavior of setting \"Tasks added\" notifications to `false`. Returns the updated project record.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_members_for_project_with_http_info(body, project_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProjectGidAddMembersBody body: Information about the members being added. (required)
        :param str project_gid: Globally unique identifier for the project. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: ProjectResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'project_gid', 'opt_fields']  # noqa: E501
        all_params.append('async_req')
        all_params.append('header_params')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method add_members_for_project" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `add_members_for_project`")  # noqa: E501
        # verify the required parameter 'project_gid' is set
        if ('project_gid' not in params or
                params['project_gid'] is None):
            raise ValueError("Missing the required parameter `project_gid` when calling `add_members_for_project`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'project_gid' in params:
            path_params['project_gid'] = params['project_gid']  # noqa: E501

        query_params = []
        if 'opt_fields' in params:
            query_params.append(('opt_fields', params['opt_fields']))  # noqa: E501
            collection_formats['opt_fields'] = 'csv'  # noqa: E501

        header_params = kwargs.get("header_params", {})

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json; charset=UTF-8'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json; charset=UTF-8'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oauth2']  # noqa: E501

        return self.api_client.call_api(
            '/projects/{project_gid}/addMembers', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='ProjectResponseData',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def create_project(self, body, **kwargs):  # noqa: E501
        """Create a project  # noqa: E501

        Create a new project in a workspace or team.  Every project is required to be created in a specific workspace or organization, and this cannot be changed once set. Note that you can use the `workspace` parameter regardless of whether or not it is an organization.  If the workspace for your project is an organization, you must also supply a `team` to share the project with.  Returns the full record of the newly created project.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_project(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProjectsBody body: The project to create. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: ProjectResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = kwargs.get("_return_http_data_only", True)
        if kwargs.get('async_req'):
            return self.create_project_with_http_info(body, **kwargs)  # noqa: E501
        else:
            (data) = self.create_project_with_http_info(body, **kwargs)  # noqa: E501
            return data

    def create_project_with_http_info(self, body, **kwargs):  # noqa: E501
        """Create a project  # noqa: E501

        Create a new project in a workspace or team.  Every project is required to be created in a specific workspace or organization, and this cannot be changed once set. Note that you can use the `workspace` parameter regardless of whether or not it is an organization.  If the workspace for your project is an organization, you must also supply a `team` to share the project with.  Returns the full record of the newly created project.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_project_with_http_info(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProjectsBody body: The project to create. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: ProjectResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'opt_fields']  # noqa: E501
        all_params.append('async_req')
        all_params.append('header_params')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_project" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `create_project`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'opt_fields' in params:
            query_params.append(('opt_fields', params['opt_fields']))  # noqa: E501
            collection_formats['opt_fields'] = 'csv'  # noqa: E501

        header_params = kwargs.get("header_params", {})

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json; charset=UTF-8'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json; charset=UTF-8'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oauth2']  # noqa: E501

        return self.api_client.call_api(
            '/projects', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='ProjectResponseData',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def create_project_for_team(self, body, team_gid, **kwargs):  # noqa: E501
        """Create a project in a team  # noqa: E501

        Creates a project shared with the given team.  Returns the full record of the newly created project.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_project_for_team(body, team_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param TeamGidProjectsBody body: The new project to create. (required)
        :param str team_gid: Globally unique identifier for the team. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: ProjectResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = kwargs.get("_return_http_data_only", True)
        if kwargs.get('async_req'):
            return self.create_project_for_team_with_http_info(body, team_gid, **kwargs)  # noqa: E501
        else:
            (data) = self.create_project_for_team_with_http_info(body, team_gid, **kwargs)  # noqa: E501
            return data

    def create_project_for_team_with_http_info(self, body, team_gid, **kwargs):  # noqa: E501
        """Create a project in a team  # noqa: E501

        Creates a project shared with the given team.  Returns the full record of the newly created project.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_project_for_team_with_http_info(body, team_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param TeamGidProjectsBody body: The new project to create. (required)
        :param str team_gid: Globally unique identifier for the team. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: ProjectResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'team_gid', 'opt_fields']  # noqa: E501
        all_params.append('async_req')
        all_params.append('header_params')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_project_for_team" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `create_project_for_team`")  # noqa: E501
        # verify the required parameter 'team_gid' is set
        if ('team_gid' not in params or
                params['team_gid'] is None):
            raise ValueError("Missing the required parameter `team_gid` when calling `create_project_for_team`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'team_gid' in params:
            path_params['team_gid'] = params['team_gid']  # noqa: E501

        query_params = []
        if 'opt_fields' in params:
            query_params.append(('opt_fields', params['opt_fields']))  # noqa: E501
            collection_formats['opt_fields'] = 'csv'  # noqa: E501

        header_params = kwargs.get("header_params", {})

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json; charset=UTF-8'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json; charset=UTF-8'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oauth2']  # noqa: E501

        return self.api_client.call_api(
            '/teams/{team_gid}/projects', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='ProjectResponseData',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def create_project_for_workspace(self, body, workspace_gid, **kwargs):  # noqa: E501
        """Create a project in a workspace  # noqa: E501

        Creates a project in the workspace.  If the workspace for your project is an organization, you must also supply a team to share the project with.  Returns the full record of the newly created project.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_project_for_workspace(body, workspace_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param WorkspaceGidProjectsBody body: The new project to create. (required)
        :param str workspace_gid: Globally unique identifier for the workspace or organization. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: ProjectResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = kwargs.get("_return_http_data_only", True)
        if kwargs.get('async_req'):
            return self.create_project_for_workspace_with_http_info(body, workspace_gid, **kwargs)  # noqa: E501
        else:
            (data) = self.create_project_for_workspace_with_http_info(body, workspace_gid, **kwargs)  # noqa: E501
            return data

    def create_project_for_workspace_with_http_info(self, body, workspace_gid, **kwargs):  # noqa: E501
        """Create a project in a workspace  # noqa: E501

        Creates a project in the workspace.  If the workspace for your project is an organization, you must also supply a team to share the project with.  Returns the full record of the newly created project.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_project_for_workspace_with_http_info(body, workspace_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param WorkspaceGidProjectsBody body: The new project to create. (required)
        :param str workspace_gid: Globally unique identifier for the workspace or organization. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: ProjectResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'workspace_gid', 'opt_fields']  # noqa: E501
        all_params.append('async_req')
        all_params.append('header_params')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_project_for_workspace" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `create_project_for_workspace`")  # noqa: E501
        # verify the required parameter 'workspace_gid' is set
        if ('workspace_gid' not in params or
                params['workspace_gid'] is None):
            raise ValueError("Missing the required parameter `workspace_gid` when calling `create_project_for_workspace`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'workspace_gid' in params:
            path_params['workspace_gid'] = params['workspace_gid']  # noqa: E501

        query_params = []
        if 'opt_fields' in params:
            query_params.append(('opt_fields', params['opt_fields']))  # noqa: E501
            collection_formats['opt_fields'] = 'csv'  # noqa: E501

        header_params = kwargs.get("header_params", {})

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json; charset=UTF-8'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json; charset=UTF-8'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oauth2']  # noqa: E501

        return self.api_client.call_api(
            '/workspaces/{workspace_gid}/projects', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='ProjectResponseData',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def delete_project(self, project_gid, **kwargs):  # noqa: E501
        """Delete a project  # noqa: E501

        A specific, existing project can be deleted by making a DELETE request on the URL for that project.  Returns an empty data record.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_project(project_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str project_gid: Globally unique identifier for the project. (required)
        :return: EmptyResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = kwargs.get("_return_http_data_only", True)
        if kwargs.get('async_req'):
            return self.delete_project_with_http_info(project_gid, **kwargs)  # noqa: E501
        else:
            (data) = self.delete_project_with_http_info(project_gid, **kwargs)  # noqa: E501
            return data

    def delete_project_with_http_info(self, project_gid, **kwargs):  # noqa: E501
        """Delete a project  # noqa: E501

        A specific, existing project can be deleted by making a DELETE request on the URL for that project.  Returns an empty data record.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_project_with_http_info(project_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str project_gid: Globally unique identifier for the project. (required)
        :return: EmptyResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['project_gid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('header_params')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_project" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'project_gid' is set
        if ('project_gid' not in params or
                params['project_gid'] is None):
            raise ValueError("Missing the required parameter `project_gid` when calling `delete_project`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'project_gid' in params:
            path_params['project_gid'] = params['project_gid']  # noqa: E501

        query_params = []

        header_params = kwargs.get("header_params", {})

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json; charset=UTF-8'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oauth2']  # noqa: E501

        return self.api_client.call_api(
            '/projects/{project_gid}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='EmptyResponseData',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def duplicate_project(self, project_gid, **kwargs):  # noqa: E501
        """Duplicate a project  # noqa: E501

        Creates and returns a job that will asynchronously handle the duplication.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.duplicate_project(project_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str project_gid: Globally unique identifier for the project. (required)
        :param ProjectGidDuplicateBody body: Describes the duplicate's name and the elements that will be duplicated.
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: JobResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = kwargs.get("_return_http_data_only", True)
        if kwargs.get('async_req'):
            return self.duplicate_project_with_http_info(project_gid, **kwargs)  # noqa: E501
        else:
            (data) = self.duplicate_project_with_http_info(project_gid, **kwargs)  # noqa: E501
            return data

    def duplicate_project_with_http_info(self, project_gid, **kwargs):  # noqa: E501
        """Duplicate a project  # noqa: E501

        Creates and returns a job that will asynchronously handle the duplication.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.duplicate_project_with_http_info(project_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str project_gid: Globally unique identifier for the project. (required)
        :param ProjectGidDuplicateBody body: Describes the duplicate's name and the elements that will be duplicated.
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: JobResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['project_gid', 'body', 'opt_fields']  # noqa: E501
        all_params.append('async_req')
        all_params.append('header_params')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method duplicate_project" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'project_gid' is set
        if ('project_gid' not in params or
                params['project_gid'] is None):
            raise ValueError("Missing the required parameter `project_gid` when calling `duplicate_project`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'project_gid' in params:
            path_params['project_gid'] = params['project_gid']  # noqa: E501

        query_params = []
        if 'opt_fields' in params:
            query_params.append(('opt_fields', params['opt_fields']))  # noqa: E501
            collection_formats['opt_fields'] = 'csv'  # noqa: E501

        header_params = kwargs.get("header_params", {})

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json; charset=UTF-8'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json; charset=UTF-8'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oauth2']  # noqa: E501

        return self.api_client.call_api(
            '/projects/{project_gid}/duplicate', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='JobResponseData',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_project(self, project_gid, **kwargs):  # noqa: E501
        """Get a project  # noqa: E501

        Returns the complete project record for a single project.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_project(project_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str project_gid: Globally unique identifier for the project. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: ProjectResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = kwargs.get("_return_http_data_only", True)
        if kwargs.get('async_req'):
            return self.get_project_with_http_info(project_gid, **kwargs)  # noqa: E501
        else:
            (data) = self.get_project_with_http_info(project_gid, **kwargs)  # noqa: E501
            return data

    def get_project_with_http_info(self, project_gid, **kwargs):  # noqa: E501
        """Get a project  # noqa: E501

        Returns the complete project record for a single project.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_project_with_http_info(project_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str project_gid: Globally unique identifier for the project. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: ProjectResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['project_gid', 'opt_fields']  # noqa: E501
        all_params.append('async_req')
        all_params.append('header_params')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_project" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'project_gid' is set
        if ('project_gid' not in params or
                params['project_gid'] is None):
            raise ValueError("Missing the required parameter `project_gid` when calling `get_project`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'project_gid' in params:
            path_params['project_gid'] = params['project_gid']  # noqa: E501

        query_params = []
        if 'opt_fields' in params:
            query_params.append(('opt_fields', params['opt_fields']))  # noqa: E501
            collection_formats['opt_fields'] = 'csv'  # noqa: E501

        header_params = kwargs.get("header_params", {})

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json; charset=UTF-8'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oauth2']  # noqa: E501

        return self.api_client.call_api(
            '/projects/{project_gid}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='ProjectResponseData',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_projects(self, **kwargs):  # noqa: E501
        """Get multiple projects  # noqa: E501

        Returns the compact project records for some filtered set of projects. Use one or more of the parameters provided to filter the projects returned. *Note: This endpoint may timeout for large domains. Try filtering by team!*  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_projects(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int limit: Results per page. The number of objects to return per page. The value must be between 1 and 100.
        :param str offset: Offset token. An offset to the next page returned by the API. A pagination request will return an offset token, which can be used as an input parameter to the next request. If an offset is not passed in, the API will return the first page of results. 'Note: You can only pass in an offset that was returned to you via a previously paginated request.'
        :param str workspace: The workspace or organization to filter projects on.
        :param str team: The team to filter projects on.
        :param bool archived: Only return projects whose `archived` field takes on the value of this parameter.
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: ProjectResponseArray
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = kwargs.get("_return_http_data_only", True)
        if kwargs.get('async_req'):
            return self.get_projects_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.get_projects_with_http_info(**kwargs)  # noqa: E501
            return data

    def get_projects_with_http_info(self, **kwargs):  # noqa: E501
        """Get multiple projects  # noqa: E501

        Returns the compact project records for some filtered set of projects. Use one or more of the parameters provided to filter the projects returned. *Note: This endpoint may timeout for large domains. Try filtering by team!*  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_projects_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int limit: Results per page. The number of objects to return per page. The value must be between 1 and 100.
        :param str offset: Offset token. An offset to the next page returned by the API. A pagination request will return an offset token, which can be used as an input parameter to the next request. If an offset is not passed in, the API will return the first page of results. 'Note: You can only pass in an offset that was returned to you via a previously paginated request.'
        :param str workspace: The workspace or organization to filter projects on.
        :param str team: The team to filter projects on.
        :param bool archived: Only return projects whose `archived` field takes on the value of this parameter.
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: ProjectResponseArray
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['limit', 'offset', 'workspace', 'team', 'archived', 'opt_fields']  # noqa: E501
        all_params.append('async_req')
        all_params.append('header_params')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_projects" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'limit' in params:
            query_params.append(('limit', params['limit']))  # noqa: E501
        if 'offset' in params:
            query_params.append(('offset', params['offset']))  # noqa: E501
        if 'workspace' in params:
            query_params.append(('workspace', params['workspace']))  # noqa: E501
        if 'team' in params:
            query_params.append(('team', params['team']))  # noqa: E501
        if 'archived' in params:
            query_params.append(('archived', params['archived']))  # noqa: E501
        if 'opt_fields' in params:
            query_params.append(('opt_fields', params['opt_fields']))  # noqa: E501
            collection_formats['opt_fields'] = 'csv'  # noqa: E501

        header_params = kwargs.get("header_params", {})

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json; charset=UTF-8'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oauth2']  # noqa: E501

        return self.api_client.call_api(
            '/projects', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='ProjectResponseArray',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_projects_for_task(self, task_gid, **kwargs):  # noqa: E501
        """Get projects a task is in  # noqa: E501

        Returns a compact representation of all of the projects the task is in.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_projects_for_task(task_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str task_gid: The task to operate on. (required)
        :param int limit: Results per page. The number of objects to return per page. The value must be between 1 and 100.
        :param str offset: Offset token. An offset to the next page returned by the API. A pagination request will return an offset token, which can be used as an input parameter to the next request. If an offset is not passed in, the API will return the first page of results. 'Note: You can only pass in an offset that was returned to you via a previously paginated request.'
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: ProjectResponseArray
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = kwargs.get("_return_http_data_only", True)
        if kwargs.get('async_req'):
            return self.get_projects_for_task_with_http_info(task_gid, **kwargs)  # noqa: E501
        else:
            (data) = self.get_projects_for_task_with_http_info(task_gid, **kwargs)  # noqa: E501
            return data

    def get_projects_for_task_with_http_info(self, task_gid, **kwargs):  # noqa: E501
        """Get projects a task is in  # noqa: E501

        Returns a compact representation of all of the projects the task is in.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_projects_for_task_with_http_info(task_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str task_gid: The task to operate on. (required)
        :param int limit: Results per page. The number of objects to return per page. The value must be between 1 and 100.
        :param str offset: Offset token. An offset to the next page returned by the API. A pagination request will return an offset token, which can be used as an input parameter to the next request. If an offset is not passed in, the API will return the first page of results. 'Note: You can only pass in an offset that was returned to you via a previously paginated request.'
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: ProjectResponseArray
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['task_gid', 'limit', 'offset', 'opt_fields']  # noqa: E501
        all_params.append('async_req')
        all_params.append('header_params')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_projects_for_task" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'task_gid' is set
        if ('task_gid' not in params or
                params['task_gid'] is None):
            raise ValueError("Missing the required parameter `task_gid` when calling `get_projects_for_task`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'task_gid' in params:
            path_params['task_gid'] = params['task_gid']  # noqa: E501

        query_params = []
        if 'limit' in params:
            query_params.append(('limit', params['limit']))  # noqa: E501
        if 'offset' in params:
            query_params.append(('offset', params['offset']))  # noqa: E501
        if 'opt_fields' in params:
            query_params.append(('opt_fields', params['opt_fields']))  # noqa: E501
            collection_formats['opt_fields'] = 'csv'  # noqa: E501

        header_params = kwargs.get("header_params", {})

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json; charset=UTF-8'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oauth2']  # noqa: E501

        return self.api_client.call_api(
            '/tasks/{task_gid}/projects', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='ProjectResponseArray',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_projects_for_team(self, team_gid, **kwargs):  # noqa: E501
        """Get a team's projects  # noqa: E501

        Returns the compact project records for all projects in the team.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_projects_for_team(team_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str team_gid: Globally unique identifier for the team. (required)
        :param int limit: Results per page. The number of objects to return per page. The value must be between 1 and 100.
        :param str offset: Offset token. An offset to the next page returned by the API. A pagination request will return an offset token, which can be used as an input parameter to the next request. If an offset is not passed in, the API will return the first page of results. 'Note: You can only pass in an offset that was returned to you via a previously paginated request.'
        :param bool archived: Only return projects whose `archived` field takes on the value of this parameter.
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: ProjectResponseArray
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = kwargs.get("_return_http_data_only", True)
        if kwargs.get('async_req'):
            return self.get_projects_for_team_with_http_info(team_gid, **kwargs)  # noqa: E501
        else:
            (data) = self.get_projects_for_team_with_http_info(team_gid, **kwargs)  # noqa: E501
            return data

    def get_projects_for_team_with_http_info(self, team_gid, **kwargs):  # noqa: E501
        """Get a team's projects  # noqa: E501

        Returns the compact project records for all projects in the team.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_projects_for_team_with_http_info(team_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str team_gid: Globally unique identifier for the team. (required)
        :param int limit: Results per page. The number of objects to return per page. The value must be between 1 and 100.
        :param str offset: Offset token. An offset to the next page returned by the API. A pagination request will return an offset token, which can be used as an input parameter to the next request. If an offset is not passed in, the API will return the first page of results. 'Note: You can only pass in an offset that was returned to you via a previously paginated request.'
        :param bool archived: Only return projects whose `archived` field takes on the value of this parameter.
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: ProjectResponseArray
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['team_gid', 'limit', 'offset', 'archived', 'opt_fields']  # noqa: E501
        all_params.append('async_req')
        all_params.append('header_params')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_projects_for_team" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'team_gid' is set
        if ('team_gid' not in params or
                params['team_gid'] is None):
            raise ValueError("Missing the required parameter `team_gid` when calling `get_projects_for_team`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'team_gid' in params:
            path_params['team_gid'] = params['team_gid']  # noqa: E501

        query_params = []
        if 'limit' in params:
            query_params.append(('limit', params['limit']))  # noqa: E501
        if 'offset' in params:
            query_params.append(('offset', params['offset']))  # noqa: E501
        if 'archived' in params:
            query_params.append(('archived', params['archived']))  # noqa: E501
        if 'opt_fields' in params:
            query_params.append(('opt_fields', params['opt_fields']))  # noqa: E501
            collection_formats['opt_fields'] = 'csv'  # noqa: E501

        header_params = kwargs.get("header_params", {})

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json; charset=UTF-8'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oauth2']  # noqa: E501

        return self.api_client.call_api(
            '/teams/{team_gid}/projects', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='ProjectResponseArray',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_projects_for_workspace(self, workspace_gid, **kwargs):  # noqa: E501
        """Get all projects in a workspace  # noqa: E501

        Returns the compact project records for all projects in the workspace. *Note: This endpoint may timeout for large domains. Prefer the `/teams/{team_gid}/projects` endpoint.*  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_projects_for_workspace(workspace_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str workspace_gid: Globally unique identifier for the workspace or organization. (required)
        :param int limit: Results per page. The number of objects to return per page. The value must be between 1 and 100.
        :param str offset: Offset token. An offset to the next page returned by the API. A pagination request will return an offset token, which can be used as an input parameter to the next request. If an offset is not passed in, the API will return the first page of results. 'Note: You can only pass in an offset that was returned to you via a previously paginated request.'
        :param bool archived: Only return projects whose `archived` field takes on the value of this parameter.
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: ProjectResponseArray
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = kwargs.get("_return_http_data_only", True)
        if kwargs.get('async_req'):
            return self.get_projects_for_workspace_with_http_info(workspace_gid, **kwargs)  # noqa: E501
        else:
            (data) = self.get_projects_for_workspace_with_http_info(workspace_gid, **kwargs)  # noqa: E501
            return data

    def get_projects_for_workspace_with_http_info(self, workspace_gid, **kwargs):  # noqa: E501
        """Get all projects in a workspace  # noqa: E501

        Returns the compact project records for all projects in the workspace. *Note: This endpoint may timeout for large domains. Prefer the `/teams/{team_gid}/projects` endpoint.*  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_projects_for_workspace_with_http_info(workspace_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str workspace_gid: Globally unique identifier for the workspace or organization. (required)
        :param int limit: Results per page. The number of objects to return per page. The value must be between 1 and 100.
        :param str offset: Offset token. An offset to the next page returned by the API. A pagination request will return an offset token, which can be used as an input parameter to the next request. If an offset is not passed in, the API will return the first page of results. 'Note: You can only pass in an offset that was returned to you via a previously paginated request.'
        :param bool archived: Only return projects whose `archived` field takes on the value of this parameter.
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: ProjectResponseArray
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['workspace_gid', 'limit', 'offset', 'archived', 'opt_fields']  # noqa: E501
        all_params.append('async_req')
        all_params.append('header_params')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_projects_for_workspace" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'workspace_gid' is set
        if ('workspace_gid' not in params or
                params['workspace_gid'] is None):
            raise ValueError("Missing the required parameter `workspace_gid` when calling `get_projects_for_workspace`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'workspace_gid' in params:
            path_params['workspace_gid'] = params['workspace_gid']  # noqa: E501

        query_params = []
        if 'limit' in params:
            query_params.append(('limit', params['limit']))  # noqa: E501
        if 'offset' in params:
            query_params.append(('offset', params['offset']))  # noqa: E501
        if 'archived' in params:
            query_params.append(('archived', params['archived']))  # noqa: E501
        if 'opt_fields' in params:
            query_params.append(('opt_fields', params['opt_fields']))  # noqa: E501
            collection_formats['opt_fields'] = 'csv'  # noqa: E501

        header_params = kwargs.get("header_params", {})

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json; charset=UTF-8'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oauth2']  # noqa: E501

        return self.api_client.call_api(
            '/workspaces/{workspace_gid}/projects', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='ProjectResponseArray',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_task_counts_for_project(self, project_gid, **kwargs):  # noqa: E501
        """Get task count of a project  # noqa: E501

        Get an object that holds task count fields. **All fields are excluded by default**. You must [opt in](/docs/inputoutput-options) using `opt_fields` to get any information from this endpoint.  This endpoint has an additional [rate limit](/docs/rate-limits) and each field counts especially high against our [cost limits](/docs/rate-limits#cost-limits).  Milestones are just tasks, so they are included in the `num_tasks`, `num_incomplete_tasks`, and `num_completed_tasks` counts.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_task_counts_for_project(project_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str project_gid: Globally unique identifier for the project. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: TaskCountResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = kwargs.get("_return_http_data_only", True)
        if kwargs.get('async_req'):
            return self.get_task_counts_for_project_with_http_info(project_gid, **kwargs)  # noqa: E501
        else:
            (data) = self.get_task_counts_for_project_with_http_info(project_gid, **kwargs)  # noqa: E501
            return data

    def get_task_counts_for_project_with_http_info(self, project_gid, **kwargs):  # noqa: E501
        """Get task count of a project  # noqa: E501

        Get an object that holds task count fields. **All fields are excluded by default**. You must [opt in](/docs/inputoutput-options) using `opt_fields` to get any information from this endpoint.  This endpoint has an additional [rate limit](/docs/rate-limits) and each field counts especially high against our [cost limits](/docs/rate-limits#cost-limits).  Milestones are just tasks, so they are included in the `num_tasks`, `num_incomplete_tasks`, and `num_completed_tasks` counts.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_task_counts_for_project_with_http_info(project_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str project_gid: Globally unique identifier for the project. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: TaskCountResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['project_gid', 'opt_fields']  # noqa: E501
        all_params.append('async_req')
        all_params.append('header_params')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_task_counts_for_project" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'project_gid' is set
        if ('project_gid' not in params or
                params['project_gid'] is None):
            raise ValueError("Missing the required parameter `project_gid` when calling `get_task_counts_for_project`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'project_gid' in params:
            path_params['project_gid'] = params['project_gid']  # noqa: E501

        query_params = []
        if 'opt_fields' in params:
            query_params.append(('opt_fields', params['opt_fields']))  # noqa: E501
            collection_formats['opt_fields'] = 'csv'  # noqa: E501

        header_params = kwargs.get("header_params", {})

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json; charset=UTF-8'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oauth2']  # noqa: E501

        return self.api_client.call_api(
            '/projects/{project_gid}/task_counts', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='TaskCountResponseData',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def project_save_as_template(self, body, project_gid, **kwargs):  # noqa: E501
        """Create a project template from a project  # noqa: E501

        Creates and returns a job that will asynchronously handle the project template creation. Note that while the resulting project template can be accessed with the API, it won't be visible in the Asana UI until Project Templates 2.0 is launched in the app. See more in [this forum post](https://forum.asana.com/t/a-new-api-for-project-templates/156432).  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.project_save_as_template(body, project_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProjectGidSaveAsTemplateBody body: Describes the inputs used for creating a project template, such as the resulting project template's name, which team it should be created in. (required)
        :param str project_gid: Globally unique identifier for the project. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: JobResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = kwargs.get("_return_http_data_only", True)
        if kwargs.get('async_req'):
            return self.project_save_as_template_with_http_info(body, project_gid, **kwargs)  # noqa: E501
        else:
            (data) = self.project_save_as_template_with_http_info(body, project_gid, **kwargs)  # noqa: E501
            return data

    def project_save_as_template_with_http_info(self, body, project_gid, **kwargs):  # noqa: E501
        """Create a project template from a project  # noqa: E501

        Creates and returns a job that will asynchronously handle the project template creation. Note that while the resulting project template can be accessed with the API, it won't be visible in the Asana UI until Project Templates 2.0 is launched in the app. See more in [this forum post](https://forum.asana.com/t/a-new-api-for-project-templates/156432).  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.project_save_as_template_with_http_info(body, project_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProjectGidSaveAsTemplateBody body: Describes the inputs used for creating a project template, such as the resulting project template's name, which team it should be created in. (required)
        :param str project_gid: Globally unique identifier for the project. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: JobResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'project_gid', 'opt_fields']  # noqa: E501
        all_params.append('async_req')
        all_params.append('header_params')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method project_save_as_template" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `project_save_as_template`")  # noqa: E501
        # verify the required parameter 'project_gid' is set
        if ('project_gid' not in params or
                params['project_gid'] is None):
            raise ValueError("Missing the required parameter `project_gid` when calling `project_save_as_template`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'project_gid' in params:
            path_params['project_gid'] = params['project_gid']  # noqa: E501

        query_params = []
        if 'opt_fields' in params:
            query_params.append(('opt_fields', params['opt_fields']))  # noqa: E501
            collection_formats['opt_fields'] = 'csv'  # noqa: E501

        header_params = kwargs.get("header_params", {})

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json; charset=UTF-8'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json; charset=UTF-8'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oauth2']  # noqa: E501

        return self.api_client.call_api(
            '/projects/{project_gid}/saveAsTemplate', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='JobResponseData',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def remove_custom_field_setting_for_project(self, body, project_gid, **kwargs):  # noqa: E501
        """Remove a custom field from a project  # noqa: E501

        Removes a custom field setting from a project.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.remove_custom_field_setting_for_project(body, project_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProjectGidRemoveCustomFieldSettingBody body: Information about the custom field setting being removed. (required)
        :param str project_gid: Globally unique identifier for the project. (required)
        :return: EmptyResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = kwargs.get("_return_http_data_only", True)
        if kwargs.get('async_req'):
            return self.remove_custom_field_setting_for_project_with_http_info(body, project_gid, **kwargs)  # noqa: E501
        else:
            (data) = self.remove_custom_field_setting_for_project_with_http_info(body, project_gid, **kwargs)  # noqa: E501
            return data

    def remove_custom_field_setting_for_project_with_http_info(self, body, project_gid, **kwargs):  # noqa: E501
        """Remove a custom field from a project  # noqa: E501

        Removes a custom field setting from a project.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.remove_custom_field_setting_for_project_with_http_info(body, project_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProjectGidRemoveCustomFieldSettingBody body: Information about the custom field setting being removed. (required)
        :param str project_gid: Globally unique identifier for the project. (required)
        :return: EmptyResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'project_gid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('header_params')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method remove_custom_field_setting_for_project" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `remove_custom_field_setting_for_project`")  # noqa: E501
        # verify the required parameter 'project_gid' is set
        if ('project_gid' not in params or
                params['project_gid'] is None):
            raise ValueError("Missing the required parameter `project_gid` when calling `remove_custom_field_setting_for_project`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'project_gid' in params:
            path_params['project_gid'] = params['project_gid']  # noqa: E501

        query_params = []

        header_params = kwargs.get("header_params", {})

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json; charset=UTF-8'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json; charset=UTF-8'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oauth2']  # noqa: E501

        return self.api_client.call_api(
            '/projects/{project_gid}/removeCustomFieldSetting', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='EmptyResponseData',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def remove_followers_for_project(self, body, project_gid, **kwargs):  # noqa: E501
        """Remove followers from a project  # noqa: E501

        Removes the specified list of users from following the project, this will not affect project membership status. Returns the updated project record.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.remove_followers_for_project(body, project_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProjectGidRemoveFollowersBody body: Information about the followers being removed. (required)
        :param str project_gid: Globally unique identifier for the project. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: ProjectResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = kwargs.get("_return_http_data_only", True)
        if kwargs.get('async_req'):
            return self.remove_followers_for_project_with_http_info(body, project_gid, **kwargs)  # noqa: E501
        else:
            (data) = self.remove_followers_for_project_with_http_info(body, project_gid, **kwargs)  # noqa: E501
            return data

    def remove_followers_for_project_with_http_info(self, body, project_gid, **kwargs):  # noqa: E501
        """Remove followers from a project  # noqa: E501

        Removes the specified list of users from following the project, this will not affect project membership status. Returns the updated project record.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.remove_followers_for_project_with_http_info(body, project_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProjectGidRemoveFollowersBody body: Information about the followers being removed. (required)
        :param str project_gid: Globally unique identifier for the project. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: ProjectResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'project_gid', 'opt_fields']  # noqa: E501
        all_params.append('async_req')
        all_params.append('header_params')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method remove_followers_for_project" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `remove_followers_for_project`")  # noqa: E501
        # verify the required parameter 'project_gid' is set
        if ('project_gid' not in params or
                params['project_gid'] is None):
            raise ValueError("Missing the required parameter `project_gid` when calling `remove_followers_for_project`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'project_gid' in params:
            path_params['project_gid'] = params['project_gid']  # noqa: E501

        query_params = []
        if 'opt_fields' in params:
            query_params.append(('opt_fields', params['opt_fields']))  # noqa: E501
            collection_formats['opt_fields'] = 'csv'  # noqa: E501

        header_params = kwargs.get("header_params", {})

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json; charset=UTF-8'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json; charset=UTF-8'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oauth2']  # noqa: E501

        return self.api_client.call_api(
            '/projects/{project_gid}/removeFollowers', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='ProjectResponseData',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def remove_members_for_project(self, body, project_gid, **kwargs):  # noqa: E501
        """Remove users from a project  # noqa: E501

        Removes the specified list of users from members of the project. Returns the updated project record.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.remove_members_for_project(body, project_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProjectGidRemoveMembersBody body: Information about the members being removed. (required)
        :param str project_gid: Globally unique identifier for the project. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: ProjectResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = kwargs.get("_return_http_data_only", True)
        if kwargs.get('async_req'):
            return self.remove_members_for_project_with_http_info(body, project_gid, **kwargs)  # noqa: E501
        else:
            (data) = self.remove_members_for_project_with_http_info(body, project_gid, **kwargs)  # noqa: E501
            return data

    def remove_members_for_project_with_http_info(self, body, project_gid, **kwargs):  # noqa: E501
        """Remove users from a project  # noqa: E501

        Removes the specified list of users from members of the project. Returns the updated project record.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.remove_members_for_project_with_http_info(body, project_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProjectGidRemoveMembersBody body: Information about the members being removed. (required)
        :param str project_gid: Globally unique identifier for the project. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: ProjectResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'project_gid', 'opt_fields']  # noqa: E501
        all_params.append('async_req')
        all_params.append('header_params')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method remove_members_for_project" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `remove_members_for_project`")  # noqa: E501
        # verify the required parameter 'project_gid' is set
        if ('project_gid' not in params or
                params['project_gid'] is None):
            raise ValueError("Missing the required parameter `project_gid` when calling `remove_members_for_project`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'project_gid' in params:
            path_params['project_gid'] = params['project_gid']  # noqa: E501

        query_params = []
        if 'opt_fields' in params:
            query_params.append(('opt_fields', params['opt_fields']))  # noqa: E501
            collection_formats['opt_fields'] = 'csv'  # noqa: E501

        header_params = kwargs.get("header_params", {})

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json; charset=UTF-8'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json; charset=UTF-8'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oauth2']  # noqa: E501

        return self.api_client.call_api(
            '/projects/{project_gid}/removeMembers', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='ProjectResponseData',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def update_project(self, body, project_gid, **kwargs):  # noqa: E501
        """Update a project  # noqa: E501

        A specific, existing project can be updated by making a PUT request on the URL for that project. Only the fields provided in the `data` block will be updated; any unspecified fields will remain unchanged.  When using this method, it is best to specify only those fields you wish to change, or else you may overwrite changes made by another user since you last retrieved the task.  Returns the complete updated project record.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.update_project(body, project_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProjectsProjectGidBody body: The updated fields for the project. (required)
        :param str project_gid: Globally unique identifier for the project. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: ProjectResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = kwargs.get("_return_http_data_only", True)
        if kwargs.get('async_req'):
            return self.update_project_with_http_info(body, project_gid, **kwargs)  # noqa: E501
        else:
            (data) = self.update_project_with_http_info(body, project_gid, **kwargs)  # noqa: E501
            return data

    def update_project_with_http_info(self, body, project_gid, **kwargs):  # noqa: E501
        """Update a project  # noqa: E501

        A specific, existing project can be updated by making a PUT request on the URL for that project. Only the fields provided in the `data` block will be updated; any unspecified fields will remain unchanged.  When using this method, it is best to specify only those fields you wish to change, or else you may overwrite changes made by another user since you last retrieved the task.  Returns the complete updated project record.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.update_project_with_http_info(body, project_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProjectsProjectGidBody body: The updated fields for the project. (required)
        :param str project_gid: Globally unique identifier for the project. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: ProjectResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'project_gid', 'opt_fields']  # noqa: E501
        all_params.append('async_req')
        all_params.append('header_params')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method update_project" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `update_project`")  # noqa: E501
        # verify the required parameter 'project_gid' is set
        if ('project_gid' not in params or
                params['project_gid'] is None):
            raise ValueError("Missing the required parameter `project_gid` when calling `update_project`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'project_gid' in params:
            path_params['project_gid'] = params['project_gid']  # noqa: E501

        query_params = []
        if 'opt_fields' in params:
            query_params.append(('opt_fields', params['opt_fields']))  # noqa: E501
            collection_formats['opt_fields'] = 'csv'  # noqa: E501

        header_params = kwargs.get("header_params", {})

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json; charset=UTF-8'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json; charset=UTF-8'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oauth2']  # noqa: E501

        return self.api_client.call_api(
            '/projects/{project_gid}', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='ProjectResponseData',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



# coding: utf-8

"""
    Asana

    This is the interface for interacting with the [Asana Platform](https://developers.asana.com). Our API reference is generated from our [OpenAPI spec] (https://raw.githubusercontent.com/Asana/openapi/master/defs/asana_oas.yaml).  # noqa: E501

    OpenAPI spec version: 1.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class GoalRelationshipRequest(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'gid': 'str',
        'resource_type': 'str',
        'resource_subtype': 'str',
        'supporting_resource': 'GoalRelationshipBaseSupportingResource',
        'contribution_weight': 'float',
        'supported_goal': 'GoalRelationshipBaseSupportedGoal'
    }

    attribute_map = {
        'gid': 'gid',
        'resource_type': 'resource_type',
        'resource_subtype': 'resource_subtype',
        'supporting_resource': 'supporting_resource',
        'contribution_weight': 'contribution_weight',
        'supported_goal': 'supported_goal'
    }

    def __init__(self, gid=None, resource_type=None, resource_subtype=None, supporting_resource=None, contribution_weight=None, supported_goal=None):  # noqa: E501
        """GoalRelationshipRequest - a model defined in Swagger"""  # noqa: E501
        self._gid = None
        self._resource_type = None
        self._resource_subtype = None
        self._supporting_resource = None
        self._contribution_weight = None
        self._supported_goal = None
        self.discriminator = None
        if gid is not None:
            self.gid = gid
        if resource_type is not None:
            self.resource_type = resource_type
        if resource_subtype is not None:
            self.resource_subtype = resource_subtype
        if supporting_resource is not None:
            self.supporting_resource = supporting_resource
        if contribution_weight is not None:
            self.contribution_weight = contribution_weight
        if supported_goal is not None:
            self.supported_goal = supported_goal

    @property
    def gid(self):
        """Gets the gid of this GoalRelationshipRequest.  # noqa: E501

        Globally unique identifier of the resource, as a string.  # noqa: E501

        :return: The gid of this GoalRelationshipRequest.  # noqa: E501
        :rtype: str
        """
        return self._gid

    @gid.setter
    def gid(self, gid):
        """Sets the gid of this GoalRelationshipRequest.

        Globally unique identifier of the resource, as a string.  # noqa: E501

        :param gid: The gid of this GoalRelationshipRequest.  # noqa: E501
        :type: str
        """

        self._gid = gid

    @property
    def resource_type(self):
        """Gets the resource_type of this GoalRelationshipRequest.  # noqa: E501

        The base type of this resource.  # noqa: E501

        :return: The resource_type of this GoalRelationshipRequest.  # noqa: E501
        :rtype: str
        """
        return self._resource_type

    @resource_type.setter
    def resource_type(self, resource_type):
        """Sets the resource_type of this GoalRelationshipRequest.

        The base type of this resource.  # noqa: E501

        :param resource_type: The resource_type of this GoalRelationshipRequest.  # noqa: E501
        :type: str
        """

        self._resource_type = resource_type

    @property
    def resource_subtype(self):
        """Gets the resource_subtype of this GoalRelationshipRequest.  # noqa: E501

        The subtype of this resource. Different subtypes retain many of the same fields and behavior, but may render differently in Asana or represent resources with different semantic meaning.  # noqa: E501

        :return: The resource_subtype of this GoalRelationshipRequest.  # noqa: E501
        :rtype: str
        """
        return self._resource_subtype

    @resource_subtype.setter
    def resource_subtype(self, resource_subtype):
        """Sets the resource_subtype of this GoalRelationshipRequest.

        The subtype of this resource. Different subtypes retain many of the same fields and behavior, but may render differently in Asana or represent resources with different semantic meaning.  # noqa: E501

        :param resource_subtype: The resource_subtype of this GoalRelationshipRequest.  # noqa: E501
        :type: str
        """
        allowed_values = ["subgoal", "supporting_work"]  # noqa: E501
        if resource_subtype not in allowed_values:
            raise ValueError(
                "Invalid value for `resource_subtype` ({0}), must be one of {1}"  # noqa: E501
                .format(resource_subtype, allowed_values)
            )

        self._resource_subtype = resource_subtype

    @property
    def supporting_resource(self):
        """Gets the supporting_resource of this GoalRelationshipRequest.  # noqa: E501


        :return: The supporting_resource of this GoalRelationshipRequest.  # noqa: E501
        :rtype: GoalRelationshipBaseSupportingResource
        """
        return self._supporting_resource

    @supporting_resource.setter
    def supporting_resource(self, supporting_resource):
        """Sets the supporting_resource of this GoalRelationshipRequest.


        :param supporting_resource: The supporting_resource of this GoalRelationshipRequest.  # noqa: E501
        :type: GoalRelationshipBaseSupportingResource
        """

        self._supporting_resource = supporting_resource

    @property
    def contribution_weight(self):
        """Gets the contribution_weight of this GoalRelationshipRequest.  # noqa: E501

        The weight that the supporting resource's progress contributes to the supported goal's progress. This can only be 0 or 1.  # noqa: E501

        :return: The contribution_weight of this GoalRelationshipRequest.  # noqa: E501
        :rtype: float
        """
        return self._contribution_weight

    @contribution_weight.setter
    def contribution_weight(self, contribution_weight):
        """Sets the contribution_weight of this GoalRelationshipRequest.

        The weight that the supporting resource's progress contributes to the supported goal's progress. This can only be 0 or 1.  # noqa: E501

        :param contribution_weight: The contribution_weight of this GoalRelationshipRequest.  # noqa: E501
        :type: float
        """

        self._contribution_weight = contribution_weight

    @property
    def supported_goal(self):
        """Gets the supported_goal of this GoalRelationshipRequest.  # noqa: E501


        :return: The supported_goal of this GoalRelationshipRequest.  # noqa: E501
        :rtype: GoalRelationshipBaseSupportedGoal
        """
        return self._supported_goal

    @supported_goal.setter
    def supported_goal(self, supported_goal):
        """Sets the supported_goal of this GoalRelationshipRequest.


        :param supported_goal: The supported_goal of this GoalRelationshipRequest.  # noqa: E501
        :type: GoalRelationshipBaseSupportedGoal
        """

        self._supported_goal = supported_goal

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(GoalRelationshipRequest, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, GoalRelationshipRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
